-- mm_rigged1.lua: Rigged to Blow LVL1 [nclok1405]

-- Navpoints --
NAVPOINT_START = "mm_rigged1_$Nstart"
NAVPOINT_START_COOP = "mm_rigged1_$Nstart_coop"
NAVPOINT_CAR = "mm_rigged1_$Ncar"
NAVPOINT_GOAL = "mm_rigged1_$Ngoal"
NAVPOINT_EXPLODE1 = "mm_rigged1_$Nexplode1"
NAVPOINT_EXPLODE2 = "mm_rigged1_$Nexplode2"
NAVPOINT_EXPLODE3 = "mm_rigged1_$Nexplode3"
NAVPOINT_EXPLODE4 = "mm_rigged1_$Nexplode4"

-- Groups --
GROUP_CAR = "mm_rigged1_$Gcar"

-- Vehicles --
VEHICLE_CAR = "mm_rigged1_$Vcar"

-- Checkpoints --
CHECKPOINT_START = MISSION_START_CHECKPOINT
CHECKPOINT_NEAR_CAR = "CHECKPOINT_NEAR_CAR"

-- Constants --
MISSION_NAME = "mm_rigged1"
GOAL_RADIUS = 3
BOMB_RADIUS = 30
SAFE_DISTANCE = 30

-- Threads --
WatchSmokeAndFireThread = -1

-- Variables --
YouFuckedUp = false
FinaleTimerExpired = false

-- Main Mission Function
function mm_rigged1_start(checkpoint, is_restart)
	local cp = checkpoint

	set_mission_author("nclok1405")

	-- Fade Out
	mission_start_fade_out()

	-- Load the car
	group_create(GROUP_CAR, true)
	
	-- You fail if the car is destroyed
	on_vehicle_destroyed("mm_rigged1_you_fucked_up", VEHICLE_CAR)
	
	-- Set the car's HP
	set_max_hit_points(VEHICLE_CAR, 1600, true)
	
	-- Make the car invincible until the player enters it
	vehicle_infinite_mass(VEHICLE_CAR, true)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, true)
	turn_invulnerable(VEHICLE_CAR, false)

	-- Lock the car until the player comes near
	set_unjackable_flag(VEHICLE_CAR, true)
	
	-- Teleport
	if cp == CHECKPOINT_START then
		-- Teleport players to Penthouse Crib
		teleport_coop("highend_$HE_crib_elevator_up", "highend_$HE_crib_elevator_up")
		
		-- DEBUG TEMP
		--cp = CHECKPOINT_NEAR_CAR
		--teleport_coop(NAVPOINT_START, NAVPOINT_START_COOP)
	elseif cp == CHECKPOINT_NEAR_CAR then
		-- Checkpoint Teleport
		teleport_coop(NAVPOINT_START, NAVPOINT_START_COOP)
	end
	
	mission_start_fade_in()
	-- Fade In

	-- Guide the player to the car
	marker_add_vehicle(VEHICLE_CAR, MINIMAP_ICON_PROTECT_ACQUIRE, INGAME_EFFECT_VEHICLE_PROTECT_ACQUIRE, SYNC_ALL)
	mission_waypoint_add(VEHICLE_CAR, SYNC_ALL)

	-- Intro
	if not is_restart then
		-- Wait until the player is ready to use the phone
		while not character_is_ready_to_speak( LOCAL_PLAYER ) do
			thread_yield()
		end

		-- Player equips the phone
		cellphone_animate_start_do()

		-- Outgoing Sound
		audio_play(CELLPHONE_OUTGOING, "foley", true)
		
		delay(0.5)

		while not character_is_ready_to_speak( LOCAL_PLAYER ) do
			thread_yield()
		end

		-- Intro dialogues
		delay(1)
		mission_help_table_nag("MM_RIGGED1_INTRO_1")
		delay(7)
		mission_help_table_nag("MM_RIGGED1_INTRO_2")
		delay(7)
		mission_help_table_nag("MM_RIGGED1_INTRO_3")
		delay(7)
		mission_help_table_nag("MM_RIGGED1_INTRO_4")
		delay(7)
		
		-- Player unequips the phone
		cellphone_animate_stop_do()
	end
	
	-- Instruct the player to find and enter the car
	mission_help_table("MM_RIGGED1_ENTER")
	
	-- Wait until the player gets near the car
	while not mm_rigged1_is_either_player_near_car() do
		thread_yield()
	end
	
	-- Unlock the car (Hopefully no NPCs try to steal it!)
	set_unjackable_flag(VEHICLE_CAR, false)
	
	-- Checkpoint
	if cp == MISSION_START_CHECKPOINT then
		mission_set_checkpoint(CHECKPOINT_NEAR_CAR, true)	-- true = Don't save current vehicle
	end
	
	-- Wait until the player gets in the car
	while not mm_rigged1_player_is_in_car() do
		thread_yield()
	end
	
	mission_waypoint_remove(SYNC_ALL)
	
	-- The car is no longer invincible
	vehicle_infinite_mass(VEHICLE_CAR, false)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, false)
	turn_vulnerable(VEHICLE_CAR)

	-- Show warning message on damage
	on_damage("mm_rigged1_on_car_damaged", VEHICLE_CAR, 0.99)

	-- Watch the smoke and fire status
	WatchSmokeAndFireThread = thread_new("mm_rigged1_watch_smoke_and_fire")

	-- Show the car's HP
	damage_indicator_on(0, VEHICLE_CAR, 0, "", get_max_hit_points(VEHICLE_CAR))
	
	-- Main Loop
	local player_exited_vehicle = false
	local goal_navpoint_active = false
	
	while true do
		if YouFuckedUp then
			-- Car dead
			break
		elseif get_dist(VEHICLE_CAR, NAVPOINT_GOAL) < GOAL_RADIUS then
			-- Reached the goal
			break
		elseif not mm_rigged1_player_is_in_car() then
			if not player_exited_vehicle then
				-- If the player exited the car, remove the goal marker and add a marker to the car
				marker_remove_navpoint(NAVPOINT_GOAL, SYNC_ALL)
				marker_add_vehicle(VEHICLE_CAR, MINIMAP_ICON_PROTECT_ACQUIRE, INGAME_EFFECT_VEHICLE_PROTECT_ACQUIRE, SYNC_ALL)
				mission_waypoint_remove(SYNC_ALL)
				mission_waypoint_add(VEHICLE_CAR, SYNC_ALL)
				mission_help_table_nag("MM_RIGGED1_REENTER")
				goal_navpoint_active = false
				player_exited_vehicle = true
			end
		elseif not goal_navpoint_active then
			-- Player (re)entered the car, so add a marker to the goal
			marker_remove_vehicle(VEHICLE_CAR, SYNC_ALL)
			marker_add_navpoint(NAVPOINT_GOAL, MINIMAP_ICON_LOCATION, INGAME_EFFECT_LOCATION, SYNC_ALL)
			mission_waypoint_remove(SYNC_ALL)
			mission_waypoint_add(NAVPOINT_GOAL, SYNC_ALL)
			mission_help_table("MM_RIGGED1_GO")
			player_exited_vehicle = false
			goal_navpoint_active = true
		end
		
		thread_yield()
	end
	
	if YouFuckedUp then
		-- Car destroyed
		if WatchSmokeAndFireThread ~= -1 then
			thread_kill(WatchSmokeAndFireThread)
			WatchSmokeAndFireThread = -1
		end
		mission_waypoint_remove(SYNC_ALL)
		marker_remove_navpoint(NAVPOINT_GOAL, SYNC_ALL)
		marker_remove_vehicle(VEHICLE_CAR, SYNC_ALL)
		damage_indicator_off(0)

		delay(3)
		
		while not character_is_ready_to_speak( LOCAL_PLAYER ) do
			thread_yield()
		end

		-- Have the phone ring
		audio_play(CELLPHONE_INCOMING, "foley", true)

		-- Wait until the player answers the phone
		while not character_is_ready_to_speak( LOCAL_PLAYER ) do
			thread_yield()
		end

		-- Player equips the phone
		cellphone_animate_start_do()

		delay(0.5)
		
		while not character_is_ready_to_speak( LOCAL_PLAYER ) do
			thread_yield()
		end

		delay(1)
		mission_help_table_nag("MM_RIGGED1_FAIL")
		delay(4)
		
		-- Player unequips the phone
		cellphone_animate_stop_do()

		-- Correctly showing mission failure text requires modified mission_help.xtbl
		-- For now, I'll just kill the player
		character_kill(LOCAL_PLAYER)
		if coop_is_active() then
			character_kill(REMOTE_PLAYER)
		end
		
		return
	end
	
	-- The car reached the goal
	damage_indicator_off(0)
	mission_waypoint_remove(SYNC_ALL)
	marker_remove_navpoint(NAVPOINT_GOAL, SYNC_ALL)
	marker_remove_vehicle(VEHICLE_CAR, SYNC_ALL)

	-- Make the car invincible
	if WatchSmokeAndFireThread ~= -1 then
		thread_kill(WatchSmokeAndFireThread)
		WatchSmokeAndFireThread = -1
	end
	on_vehicle_destroyed("", VEHICLE_CAR)
	on_damage("", VEHICLE_CAR)
	vehicle_infinite_mass(VEHICLE_CAR, true)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, true)
	turn_invulnerable(VEHICLE_CAR, false)
	set_unjackable_flag(VEHICLE_CAR, true)
	
	vehicle_stop(VEHICLE_CAR, false)
	audio_play_for_navpoint("SFX_DON_BOMBPLANTED", NAVPOINT_GOAL, "foley")
	
	-- Make players exit the car
	if character_is_in_vehicle(LOCAL_PLAYER, VEHICLE_CAR) then
		vehicle_exit(LOCAL_PLAYER)
	end
	if coop_is_active() and character_is_in_vehicle(REMOTE_PLAYER, VEHICLE_CAR) then
		vehicle_exit(REMOTE_PLAYER)
	end

	-- Time to escape!
	mission_help_table("MM_RIGGED1_ESCAPE")
	minimap_icon_add_vehicle_radius(VEHICLE_CAR, "map_other_blip_human", BOMB_RADIUS)
	
	hud_timer_set(0, 15 * 1000, "mm_rigged1_finale_timer_ended")
	while not FinaleTimerExpired do
		thread_yield()
		
		-- If everyone escaped to safe distance, explode quickly
		local p1safe = (get_dist(LOCAL_PLAYER, VEHICLE_CAR) >= BOMB_RADIUS + SAFE_DISTANCE)
		local p2safe = (not coop_is_active()) or (get_dist(REMOTE_PLAYER, VEHICLE_CAR) >= BOMB_RADIUS + SAFE_DISTANCE)
		if p1safe and p2safe then
			break
		end
	end
	hud_timer_stop(0)
	
	minimap_icon_remove_vehicle(VEHICLE_CAR)
	
	-- Explode the car
	vehicle_infinite_mass(VEHICLE_CAR, false)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, false)
	turn_vulnerable(VEHICLE_CAR)

	explosion_create("Car Bomb Big", VEHICLE_CAR, "", true)
	explosion_create("Tanker", VEHICLE_CAR, "", true)
	
	explosion_create("Tanker", NAVPOINT_GOAL, "", true)
	
	explosion_create("Big Bang", NAVPOINT_EXPLODE1, "", true)
	explosion_create("Tanker", NAVPOINT_EXPLODE2, "", true)
	explosion_create("Medium Bang", NAVPOINT_EXPLODE3, "", true)
	explosion_create("Big Bang", NAVPOINT_EXPLODE4, "", true)
	
	if get_dist(LOCAL_PLAYER, VEHICLE_CAR) < BOMB_RADIUS then
		explosion_create("Car Bomb Big", LOCAL_PLAYER, "", true)
	end
	if coop_is_active() and (get_dist(REMOTE_PLAYER, VEHICLE_CAR) < BOMB_RADIUS) then
		explosion_create("Car Bomb Big", REMOTE_PLAYER, "", true)
	end
	
	-- Wait until all players are ready
	while not mm_rigged1_is_both_players_ready() do
		thread_yield()
	end
	
	-- Mission Completed
	delay(2)
	mission_end_success(MISSION_NAME)
end

-- Returns true if both players are okay
function mm_rigged1_is_both_players_ready()
	local p1ready = character_is_ready(LOCAL_PLAYER)
	local p2ready = true
	if coop_is_active() then
		p2ready = character_is_ready(REMOTE_PLAYER)
	end
	return (p1ready and p2ready)
end

-- Returns true if either player is near the car
function mm_rigged1_is_either_player_near_car()
	if get_dist(LOCAL_PLAYER, VEHICLE_CAR) < 10 then
		return true
	end
	if coop_is_active() and (get_dist(REMOTE_PLAYER, VEHICLE_CAR) < 10) then
		return true
	end
	return false
end

-- Returns true if either player is in the car
function mm_rigged1_player_is_in_car()
	if character_is_in_vehicle(LOCAL_PLAYER, VEHICLE_CAR) and character_is_in_a_driver_seat(LOCAL_PLAYER) then
		return true
	end
	if coop_is_active() and character_is_in_vehicle(REMOTE_PLAYER, VEHICLE_CAR) and character_is_in_a_driver_seat(REMOTE_PLAYER) then
		return true
	end
	return false
end

-- Called when the car gets destroyed
function mm_rigged1_you_fucked_up()
	on_vehicle_destroyed("", VEHICLE_CAR)
	YouFuckedUp = true
end

-- Damage warning
function mm_rigged1_on_car_damaged()
	on_damage("", VEHICLE_CAR)
	mission_help_table_nag("MM_RIGGED1_WARNING_1")
end

-- Watch smoke and fire status of the car
function mm_rigged1_watch_smoke_and_fire()
	local smoke
	local fire
	local smoke_warning_displayed = false
	
	while true do
		if (not vehicle_exists(VEHICLE_CAR)) or vehicle_is_destroyed(VEHICLE_CAR) then
			return	-- exit if the vehicle is destroyed
		end

		smoke, fire = vehicle_get_smoke_and_fire_state(VEHICLE_CAR)
		
		-- Show a warning when the car starts smoking
		if smoke and (not smoke_warning_displayed) then
			mission_help_table_nag("MM_RIGGED1_WARNING_2")
			smoke_warning_displayed = true
		end
		
		-- If the car catches fire, we're doomed
		if fire then
			vehicle_detonate(VEHICLE_CAR)
			return
		end
		
		thread_yield()
	end
end

-- Called when the finale timer ended
function mm_rigged1_finale_timer_ended()
	FinaleTimerExpired = true
end

-- Called when the mission ends with a success
function mm_rigged1_success()
end

-- Cleanup
function mm_rigged1_cleanup()
	if WatchSmokeAndFireThread ~= -1 then
		thread_kill(WatchSmokeAndFireThread)
		WatchSmokeAndFireThread = -1
	end

	on_vehicle_destroyed("", VEHICLE_CAR)
	on_damage("", VEHICLE_CAR)

	group_destroy(GROUP_CAR)

	objective_text_clear(0)
	damage_indicator_off(0)
	hud_timer_stop(0)
	mission_waypoint_remove(SYNC_ALL)
end
