-- mm_rigged2.lua: Rigged to Blow LVL2 [nclok1405]

-- Navpoints --
NAVPOINT_LOCAL_START = "mm_rigged2_$Nstart"
NAVPOINT_REMOTE_START = "mm_rigged2_$Ncoop_start"
NAVPOINT_CAR = "mm_rigged2_$Ncar"
NAVPOINT_GOAL = "mm_rigged2_$Ngoal"

-- Groups --
GROUP_COURTESY_CAR = "mm_rigged2_$Gcourtesy_car"
GROUP_COURTESY_CAR_COOP = "mm_rigged2_$Gcourtesy_car_coop"
GROUP_CAR = "mm_rigged2_$Gcar"

-- Vehicles --
VEHICLE_CAR = "mm_rigged2_$Vcar"

-- Checkpoints --
CHECKPOINT_START = MISSION_START_CHECKPOINT
CHECKPOINT_NEAR_CAR = "CHECKPOINT_NEAR_CAR"

-- Constants --
MISSION_NAME = "mm_rigged2"
GOAL_RADIUS = 5

-- Threads --
IntroDialogueThread = -1
WatchSmokeAndFireThread = -1

-- Variables --
YouFuckedUp = false

-- Main Mission Function
function mm_rigged2_start(checkpoint, is_restart)
	local cp = checkpoint

	set_mission_author("nclok1405")

	-- Fade Out
	mission_start_fade_out()

	-- Load the car
	group_create(GROUP_CAR, true)

	group_create(GROUP_COURTESY_CAR, true)	-- Load the courtesy car
	if coop_is_active() then
		group_create(GROUP_COURTESY_CAR_COOP, true)	-- Load the courtesy car for co-op
	end

	-- You fail if the car is destroyed
	on_vehicle_destroyed("mm_rigged2_you_fucked_up", VEHICLE_CAR)
	
	-- Set the car's HP
	set_max_hit_points(VEHICLE_CAR, 500, true)
	
	-- Make the car invincible until the player enters it
	vehicle_infinite_mass(VEHICLE_CAR, true)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, true)
	turn_invulnerable(VEHICLE_CAR, false)

	-- Lock the car until the player comes near
	set_unjackable_flag(VEHICLE_CAR, true)
	
	if cp == CHECKPOINT_START then
		-- Teleport players
		teleport_coop(NAVPOINT_LOCAL_START, NAVPOINT_REMOTE_START)
	end
	
	mission_start_fade_in()
	-- Fade In

	-- Guide the player to the car
	marker_add_vehicle(VEHICLE_CAR, MINIMAP_ICON_PROTECT_ACQUIRE, INGAME_EFFECT_VEHICLE_PROTECT_ACQUIRE, SYNC_ALL)
	mission_waypoint_add(VEHICLE_CAR, SYNC_ALL)

	-- Intro
	if not is_restart then
		IntroDialogueThread = thread_new("mm_rigged2_intro_thread")
	end
	
	-- Start time limit
	if cp == CHECKPOINT_START then
		hud_timer_set(0, 120 * 1000, "mm_rigged2_bomb_timer_ended")
	else
		hud_timer_set(0, 90 * 1000, "mm_rigged2_bomb_timer_ended")
	end
	
	-- Wait until the player gets near the car
	while (not mm_rigged2_is_either_player_near_car()) and (not YouFuckedUp) do
		thread_yield()
	end
	
	if YouFuckedUp then
		-- Car dead
		mm_rigged2_player_fucked_up()
		return
	end
	
	-- Unlock the car (Hopefully no NPCs try to steal it!)
	set_unjackable_flag(VEHICLE_CAR, false)
	
	-- Checkpoint
	if cp == MISSION_START_CHECKPOINT then
		mission_set_checkpoint(CHECKPOINT_NEAR_CAR, true)	-- true = Don't save current vehicle
	end
	
	-- Wait until the player gets in the car
	while (not mm_rigged2_player_is_in_car()) and (not YouFuckedUp) do
		thread_yield()
	end

	mission_waypoint_remove(SYNC_ALL)

	if YouFuckedUp then
		-- Car dead
		mm_rigged2_player_fucked_up()
		return
	end
	
	-- The car is no longer invincible
	vehicle_infinite_mass(VEHICLE_CAR, false)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, false)
	turn_vulnerable(VEHICLE_CAR)

	-- Show warning message on damage
	on_damage("mm_rigged2_on_car_damaged", VEHICLE_CAR, 0.99)

	-- Watch the smoke and fire status
	WatchSmokeAndFireThread = thread_new("mm_rigged2_watch_smoke_and_fire")

	-- Show the car's HP
	damage_indicator_on(0, VEHICLE_CAR, 0, "", get_max_hit_points(VEHICLE_CAR))
	
	-- Main Loop
	local player_exited_vehicle = false
	local goal_navpoint_active = false
	
	while true do
		if YouFuckedUp then
			-- Car dead
			break
		elseif get_dist(VEHICLE_CAR, NAVPOINT_GOAL) < GOAL_RADIUS then
			-- Reached the goal
			break
		elseif not mm_rigged2_player_is_in_car() then
			if not player_exited_vehicle then
				-- If the player exited the car, remove the goal marker and add a marker to the car
				marker_remove_navpoint(NAVPOINT_GOAL, SYNC_ALL)
				marker_add_vehicle(VEHICLE_CAR, MINIMAP_ICON_PROTECT_ACQUIRE, INGAME_EFFECT_VEHICLE_PROTECT_ACQUIRE, SYNC_ALL)
				mission_waypoint_remove(SYNC_ALL)
				mission_waypoint_add(VEHICLE_CAR, SYNC_ALL)
				mission_help_table_nag("MM_RIGGED2_REENTER")
				goal_navpoint_active = false
				player_exited_vehicle = true
			end
		elseif not goal_navpoint_active then
			-- Player (re)entered the car, so add a marker to the goal
			marker_remove_vehicle(VEHICLE_CAR, SYNC_ALL)
			marker_add_navpoint(NAVPOINT_GOAL, MINIMAP_ICON_LOCATION, INGAME_EFFECT_LOCATION, SYNC_ALL)
			mission_waypoint_remove(SYNC_ALL)
			mission_waypoint_add(NAVPOINT_GOAL, SYNC_ALL)
			mission_help_table("MM_RIGGED2_GO")
			player_exited_vehicle = false
			goal_navpoint_active = true
		end
		
		thread_yield()
	end
	
	if YouFuckedUp then
		-- Car dead
		mm_rigged2_player_fucked_up()
		return
	end
	
	-- The car reached Saint's Row's Rim Jobs
	marker_remove_navpoint(NAVPOINT_GOAL, SYNC_ALL)
	on_damage("", VEHICLE_CAR)
	hud_timer_stop(0)
	damage_indicator_off(0)
	mission_waypoint_remove(SYNC_ALL)

	vehicle_stop(VEHICLE_CAR, false)

	player_controls_disable(LOCAL_PLAYER)
	if coop_is_active() then
		player_controls_disable(REMOTE_PLAYER)
	end
	fade_out(1)
	fade_out_block()
	
	audio_play("SFX_CAR_UPGRADE", "foley", true)
	set_max_hit_points(VEHICLE_CAR, 4000, true)
	vehicle_repair(VEHICLE_CAR)
	
	fade_in(1)
	fade_in_block()
	player_controls_enable(LOCAL_PLAYER)
	if coop_is_active() then
		player_controls_enable(REMOTE_PLAYER)
	end
	
	-- Main Loop for return trip
	player_exited_vehicle = false
	goal_navpoint_active = false
	
	while true do
		if YouFuckedUp then
			-- Car dead
			break
		elseif get_dist(VEHICLE_CAR, NAVPOINT_CAR) < GOAL_RADIUS then
			-- Reached the goal
			-- I once planned that the car must be >= 99% HP before the mission can end, but got cut because that's too annoying
			break
		elseif not mm_rigged2_player_is_in_car() then
			if not player_exited_vehicle then
				-- If the player exited the car, remove the goal marker and add a marker to the car
				-- BUG: Sometimes exiting and reentering the car crashes the game,
				--      and the chance of crash is much higher here. I have no idea why.
				marker_remove_navpoint(NAVPOINT_CAR, SYNC_ALL)
				marker_add_vehicle(VEHICLE_CAR, MINIMAP_ICON_PROTECT_ACQUIRE, INGAME_EFFECT_VEHICLE_PROTECT_ACQUIRE, SYNC_ALL)
				mission_waypoint_remove(SYNC_ALL)
				mission_waypoint_add(VEHICLE_CAR, SYNC_ALL)
				mission_help_table_nag("MM_RIGGED2_REENTER")
				goal_navpoint_active = false
				player_exited_vehicle = true
			end
		elseif not goal_navpoint_active then
			-- Player (re)entered the car, so add a marker to the goal
			marker_remove_vehicle(VEHICLE_CAR, SYNC_ALL)
			marker_add_navpoint(NAVPOINT_CAR, MINIMAP_ICON_LOCATION, INGAME_EFFECT_LOCATION, SYNC_ALL)
			mission_waypoint_remove(SYNC_ALL)
			mission_waypoint_add(NAVPOINT_CAR, SYNC_ALL)
			mission_help_table("MM_RIGGED2_RETURN")
			player_exited_vehicle = false
			goal_navpoint_active = true
		end
		
		thread_yield()
	end
	
	if YouFuckedUp then
		-- Car dead
		mm_rigged2_player_fucked_up()
		return
	end

	-- Reached the parking area
	mission_waypoint_remove(SYNC_ALL)
	marker_remove_navpoint(NAVPOINT_CAR, SYNC_ALL)

	vehicle_stop(VEHICLE_CAR, false)

	vehicle_infinite_mass(VEHICLE_CAR, true)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, true)
	turn_invulnerable(VEHICLE_CAR, false)

	-- Make players exit the car
	if character_is_in_vehicle(LOCAL_PLAYER, VEHICLE_CAR) then
		vehicle_exit(LOCAL_PLAYER)
	end
	if coop_is_active() and character_is_in_vehicle(REMOTE_PLAYER, VEHICLE_CAR) then
		vehicle_exit(REMOTE_PLAYER)
	end

	set_unjackable_flag(VEHICLE_CAR, true)
	
	-- Mission Completed
	mission_end_success(MISSION_NAME)
end

-- Intro dialogues
function mm_rigged2_intro_thread()
	-- Have the phone ring
	audio_play(CELLPHONE_INCOMING, "foley", true)

	delay(0.5)

	-- Intro dialogues
	delay(1)
	mission_help_table_nag("MM_RIGGED2_INTRO_1")
	delay(7)
	mission_help_table_nag("MM_RIGGED2_INTRO_2")
	delay(7)

	-- Instruct the player to find and enter the car
	mission_help_table("MM_RIGGED2_ENTER")
end

-- Car destroyed
function mm_rigged2_player_fucked_up()
	mission_waypoint_remove(SYNC_ALL)
	marker_remove_vehicle(VEHICLE_CAR, SYNC_ALL)
	marker_remove_navpoint(NAVPOINT_GOAL, SYNC_ALL)
	marker_remove_navpoint(NAVPOINT_CAR, SYNC_ALL)
	hud_timer_stop(0)
	
	if WatchSmokeAndFireThread ~= -1 then
		thread_kill(WatchSmokeAndFireThread)
		WatchSmokeAndFireThread = -1
	end

	delay(3)
	
	while not character_is_ready_to_speak( LOCAL_PLAYER ) do
		thread_yield()
	end

	-- Have the phone ring
	audio_play(CELLPHONE_INCOMING, "foley", true)

	-- Wait until the player answers the phone
	while not character_is_ready_to_speak( LOCAL_PLAYER ) do
		thread_yield()
	end

	-- Player equips the phone
	cellphone_animate_start_do()

	delay(0.5)
	
	while not character_is_ready_to_speak( LOCAL_PLAYER ) do
		thread_yield()
	end

	delay(1)
	mission_help_table_nag("MM_RIGGED2_FAIL")
	delay(4)
	
	-- Player unequips the phone
	cellphone_animate_stop_do()

	-- Correctly showing mission failure text requires modified mission_help.xtbl
	-- For now, I'll just kill the player
	character_kill(LOCAL_PLAYER)
	if coop_is_active() then
		character_kill(REMOTE_PLAYER)
	end
end

-- Returns true if both players are okay
function mm_rigged2_is_both_players_ready()
	local p1ready = character_is_ready(LOCAL_PLAYER)
	local p2ready = true
	if coop_is_active() then
		p2ready = character_is_ready(REMOTE_PLAYER)
	end
	return (p1ready and p2ready)
end

-- Returns true if either player is near the car
function mm_rigged2_is_either_player_near_car()
	if get_dist(LOCAL_PLAYER, VEHICLE_CAR) < 10 then
		return true
	end
	if coop_is_active() and (get_dist(REMOTE_PLAYER, VEHICLE_CAR) < 10) then
		return true
	end
	return false
end

-- Returns true if either player is in the car
function mm_rigged2_player_is_in_car()
	if character_is_in_vehicle(LOCAL_PLAYER, VEHICLE_CAR) and character_is_in_a_driver_seat(LOCAL_PLAYER) then
		return true
	end
	if coop_is_active() and character_is_in_vehicle(REMOTE_PLAYER, VEHICLE_CAR) and character_is_in_a_driver_seat(REMOTE_PLAYER) then
		return true
	end
	return false
end

-- Called when the car gets destroyed
function mm_rigged2_you_fucked_up()
	on_vehicle_destroyed("", VEHICLE_CAR)
	YouFuckedUp = true
end

-- Damage warning
function mm_rigged2_on_car_damaged()
	on_damage("", VEHICLE_CAR)
	mission_help_table_nag("MM_RIGGED2_WARNING_1")
end

-- Watch smoke and fire status of the car
function mm_rigged2_watch_smoke_and_fire()
	local smoke
	local fire
	local smoke_warning_displayed = false
	
	while true do
		if (not vehicle_exists(VEHICLE_CAR)) or vehicle_is_destroyed(VEHICLE_CAR) then
			return	-- exit if the vehicle is destroyed
		end

		smoke, fire = vehicle_get_smoke_and_fire_state(VEHICLE_CAR)
		
		-- Show a warning when the car starts smoking
		if smoke and (not smoke_warning_displayed) then
			mission_help_table_nag("MM_RIGGED2_WARNING_2")
			smoke_warning_displayed = true
		end
		
		-- If the car catches fire, we're doomed
		if fire then
			vehicle_detonate(VEHICLE_CAR)
			return
		end
		
		thread_yield()
	end
end

-- Called when the bomb timer ended
function mm_rigged2_bomb_timer_ended()
	on_vehicle_destroyed("", VEHICLE_CAR)
	vehicle_infinite_mass(VEHICLE_CAR, false)
	vehicle_prevent_explosion_fling(VEHICLE_CAR, false)
	turn_vulnerable(VEHICLE_CAR)
	vehicle_detonate(VEHICLE_CAR)
	YouFuckedUp = true
end

-- Called when the mission ends with a success
function mm_rigged2_success()
end

-- Cleanup
function mm_rigged2_cleanup()
	if IntroDialogueThread ~= -1 then
		thread_kill(IntroDialogueThread)
		IntroDialogueThread = -1
	end
	if WatchSmokeAndFireThread ~= -1 then
		thread_kill(WatchSmokeAndFireThread)
		WatchSmokeAndFireThread = -1
	end

	on_vehicle_destroyed("", VEHICLE_CAR)
	on_damage("", VEHICLE_CAR)

	group_destroy(GROUP_CAR)

	release_to_world(GROUP_COURTESY_CAR)
	if coop_is_active() then
		release_to_world(GROUP_COURTESY_CAR_COOP)
	end

	objective_text_clear(0)
	damage_indicator_off(0)
	hud_timer_stop(0)
	mission_waypoint_remove(SYNC_ALL)
end
